local addonName, addonTab = ... --[[	addonName = The name of this addon as in the TOC file and folder name. addonTab = The shared addon table between the Lua files of this addon.]]
local L = addonTab.L --{ Localization
addonTab.version = GetAddOnMetadata(addonName, "Version")
addonTab.WoWClient = {
	Retail = (WOW_PROJECT_ID == WOW_PROJECT_MAINLINE),
	Wrath = (WOW_PROJECT_ID == WOW_PROJECT_WRATH_CLASSIC),
	TBCC = (WOW_PROJECT_ID == WOW_PROJECT_BURNING_CRUSADE_CLASSIC),
	Vanilla = (WOW_PROJECT_ID == WOW_PROJECT_CLASSIC),
}

--{ Defaults }--
if not (addonTab.defaults) then
	addonTab.defaults = {}
end

addonTab.defaults.DejaMark = {
    maskcolor = {0, 0.75, 1, 0.33}, --{ Default color of the mask when checked.
	flarePlacedCheck = false, --{ Default state of the masked Overlay when checked.
}

--{ SavedVariables Loader }--
if not (addonTab.loader) then
	addonTab.loader = CreateFrame("Frame")
		addonTab.loader:RegisterEvent("ADDON_LOADED")
		addonTab.loader:SetScript("OnEvent", function(self, event, ...)
			if (event == "ADDON_LOADED" and ... == addonName) then
				addonTab.loader[event](self, ...)
			end
		end)

	function addonTab.loader:ADDON_LOADED(addonName)
		if not (_G[addonName.."DBPC"]) then
			print(addonName..": First time use, creating saved variables")
			--{ Create the empty saved variables table }--
			_G[addonName.."DBPC"] = {}
			--{ Copy the defaults table into the saved variables table }--
			_G[addonName.."DBPC"] = addonTab.defaults
		end
		--{ Check for missing entries in the saved variables table }--
		for k,v in pairs(addonTab.defaults) do
			if not (_G[addonName.."DBPC"][k]) then
				print(addonName..": Missing entry in saved variables, adding it")
				_G[addonName.."DBPC"][k] = v
			end
		end
		--{ Check for obsolete entries in the saved variables table }--
		for k,v in pairs(_G[addonName.."DBPC"]) do
			if not (addonTab.defaults[k]) then
				print(addonName..": Obsolete entry in saved variables, removing it", k, v)
				_G[addonName.."DBPC"][k] = nil
			end
		end
		--{ Copy the saved variables table into the addon's table }--
		addonTab.db = _G[addonName.."DBPC"]
	end
end

--{ DejaMark Options Panel }--
	local DejaMarkOptionsPanel = CreateFrame("Frame", addonName.."OptionsPanel", UIParent)
	if (addonTab.WoWClient.Retail) then
		Settings.RegisterAddOnCategory( Settings.RegisterCanvasLayoutCategory(DejaMarkOptionsPanel, addonName) )
	else
		DejaMarkOptionsPanel.name = "DejaMark"
		InterfaceOptions_AddCategory(DejaMarkOptionsPanel)
	end

	DejaMarkOptionsPanel:RegisterEvent("PLAYER_LOGOUT")
	DejaMarkOptionsPanel:RegisterEvent("PLAYER_LEAVING_WORLD")

	DejaMarkOptionsPanel.title = DejaMarkOptionsPanel:CreateFontString("DejaMarkOptionsPanelTitle", "ARTWORK", "GameFontNormalLarge")
		DejaMarkOptionsPanel.title:SetPoint("TOPLEFT", DejaMarkOptionsPanel, "TOPLEFT", 16, -16)
		DejaMarkOptionsPanel.title:SetFont("Fonts\\MORPHEUS.TTF", 32, "THINOUTLINE")
		DejaMarkOptionsPanel.title:SetTextColor(0, 192, 255, 1)
		DejaMarkOptionsPanel.title:SetText(addonName)

	DejaMarkOptionsPanel.version = DejaMarkOptionsPanel:CreateFontString("DejaMarkOptionsPanelVersion", "ARTWORK", "GameFontNormal")
		DejaMarkOptionsPanel.version:SetPoint("BOTTOMRIGHT", DejaMarkOptionsPanel, "BOTTOMRIGHT", -16, 32)
		DejaMarkOptionsPanel.version:SetText(addonName..": "..addonTab.version)
		DejaMarkOptionsPanel.version:SetJustifyH("RIGHT")
		DejaMarkOptionsPanel.version:SetTextColor(0, 0.75, 1, 1)

	DejaMarkOptionsPanel.WoWClient = DejaMarkOptionsPanel:CreateFontString("DejaMarkOptionsPanelWoWClient", "ARTWORK", "GameFontNormal")
		DejaMarkOptionsPanel.WoWClient:SetPoint("BOTTOMRIGHT", DejaMarkOptionsPanel, "BOTTOMRIGHT", -16, 16)
		DejaMarkOptionsPanel.WoWClient:SetText("WoW Client: "..(addonTab.WoWClient.Retail and "Retail" or addonTab.WoWClient.Wrath and "Wrath" or addonTab.WoWClient.TBCC and "TBCC" or addonTab.WoWClient.Vanilla and "Vanilla" or "Unknown"))
		DejaMarkOptionsPanel.WoWClient:SetJustifyH("RIGHT")
		DejaMarkOptionsPanel.WoWClient:SetTextColor(0, 0.75, 1, 1)

	DejaMarkOptionsPanel:SetScript("OnEvent", function(self, event, ...)
		if (event == "PLAYER_LOGOUT" or event == "PLAYER_LEAVING_WORLD") then
			addonTab.db.DejaMark.flarePlacedCheck = false
			_G[addonName.."DBPC"] = addonTab.db
		end
	end)

	DejaMarkOptionsPanel:SetScript("OnShow", function(self)
		addonTab.toggleframe:Disable()
		local DejaMarkFrame = _G["DejaMarkFrame"]
		DejaMarkFrame:SetFrameStrata("TOOLTIP")
		DejaMarkFrame:SetPoint("CENTER", DejaMarkOptionsPanel, "CENTER", 5, -5)
		DejaMarkFrame:Show()
		addonTab.db.DejaMark.flarePlacedCheck = true
	end)

	DejaMarkOptionsPanel:SetScript("OnHide", function(self)
		addonTab.toggleframe:Enable()
		DejaMarkFrame:SetFrameStrata("BACKGROUND")
		DejaMarkFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
		DejaMarkFrame:Hide()
		addonTab.db.DejaMark.flarePlacedCheck = false
	end)

--{ DejaMark Options Panel: Defaulkt Reset Button }--
local DMReset = CreateFrame("Button", "DMResetButton", DejaMarkOptionsPanel, "UIPanelButtonTemplate")
	DMReset:ClearAllPoints()
	DMReset:SetPoint("BOTTOMLEFT", 5, 5)
	DMReset:SetScale(1.0)
	DMReset:SetWidth(175)
	DMReset:SetHeight(30)
	_G[DMReset:GetName() .. "Text"]:SetText("Reset to Defaults")
	DMReset.tooltipText = 'Reset |cff00c0ffDejaMark|r to its default settings.|n|cffff0000WARNING! This will reload the UI!|r'

	DMReset:SetScript("OnClick", function(self, button, down)
		addonTab.db = addonTab.defaults
		_G[addonName.."DBPC"] = addonTab.db
		DejaMarkOptionsPanel:Hide()
		ReloadUI()
	end)

--{ DejaMark Options Panel: Color Picker Button }--
local DMColorPicker = CreateFrame("Button", "DejaMarkColorPickerButton", DejaMarkOptionsPanel, "UIPanelButtonTemplate")
	DMColorPicker:ClearAllPoints()
	DMColorPicker:SetPoint("CENTER", DejaMarkOptionsPanel, "CENTER", 0, -80)
	DMColorPicker:SetScale(1.0)
	DMColorPicker:SetWidth(175)
	DMColorPicker:SetHeight(30)
	_G[DMColorPicker:GetName() .. "Text"]:SetText("Deployed Flare Color")
	DMColorPicker.tooltipText = 'Select a custom indication color for World Marker Flares currently deployed in the field.'

	DMColorPicker:SetScript("OnClick", function(self, button, down)
		local r, g, b, a = unpack(addonTab.db.DejaMark.maskcolor)
		addonTab.ShowColorPicker(r, g, b, a, addonTab.myColorCallback) --{ Any icon texture 1 - 8 will do, just have to get the current color of any of them since they are all the same.
	end)

--{ Color Picker Functions }--
function addonTab.ShowColorPicker(r, g, b, a, changedCallback)
	ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = (a ~= nil), a;
	ColorPickerFrame.previousValues = {r, g, b, a};

	--{ Register our callback(s) here accordingly }--
	ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc =
	changedCallback, changedCallback, changedCallback;

	ColorPickerFrame:SetColorRGB(r, g, b);
	ColorPickerFrame:Hide(); --{ Need to run the OnShow handler.
	ColorPickerFrame:ClearAllPoints()
	ColorPickerFrame:SetPoint("TOP", DejaMarkOptionsPanel, "TOP", 0, -32)
	ColorPickerFrame:Show();
end

function addonTab.myColorCallback(restore)
 	local newR, newG, newB, newA;
	if restore then
		--{ User canceled, restore old values. }--
		newR, newG, newB, newA = unpack(restore);
		ColorPickerFrame:ClearAllPoints()
		ColorPickerFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	else
		--{ User accepted, set new values. }--
		newA, newR, newG, newB = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB();
	end

 	--{ Update our saved variables with the new values. }--
	addonTab.db.DejaMark.maskcolor = {newR, newG, newB, newA}

 	--{ Update our UI elements with new values. }--
 	DM_DesatMarks()
end
